# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class ValidationResponse(object):
    """
    The validation response returned when checking whether the requested user is valid.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new ValidationResponse object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param is_valid_user:
            The value to assign to the is_valid_user property of this ValidationResponse.
        :type is_valid_user: bool

        :param write_permitted_user_group_infos:
            The value to assign to the write_permitted_user_group_infos property of this ValidationResponse.
        :type write_permitted_user_group_infos: list[oci.cims.models.CmosUserGroupInfo]

        """
        self.swagger_types = {
            'is_valid_user': 'bool',
            'write_permitted_user_group_infos': 'list[CmosUserGroupInfo]'
        }
        self.attribute_map = {
            'is_valid_user': 'isValidUser',
            'write_permitted_user_group_infos': 'writePermittedUserGroupInfos'
        }
        self._is_valid_user = None
        self._write_permitted_user_group_infos = None

    @property
    def is_valid_user(self):
        """
        Gets the is_valid_user of this ValidationResponse.
        Boolean value that indicates whether the requested user is valid.


        :return: The is_valid_user of this ValidationResponse.
        :rtype: bool
        """
        return self._is_valid_user

    @is_valid_user.setter
    def is_valid_user(self, is_valid_user):
        """
        Sets the is_valid_user of this ValidationResponse.
        Boolean value that indicates whether the requested user is valid.


        :param is_valid_user: The is_valid_user of this ValidationResponse.
        :type: bool
        """
        self._is_valid_user = is_valid_user

    @property
    def write_permitted_user_group_infos(self):
        """
        Gets the write_permitted_user_group_infos of this ValidationResponse.
        Array of user groups that the user has access to for creating technical support type (`TECH`) support requests.
        Each user group is indicated by its identifier and name (`userGroupId` and `userGroupName`).
        Note: The Customer User Administrator (CUA) can manage user groups by name using
        `My Oracle Cloud Support portal`__.

        __ https://support.oracle.com


        :return: The write_permitted_user_group_infos of this ValidationResponse.
        :rtype: list[oci.cims.models.CmosUserGroupInfo]
        """
        return self._write_permitted_user_group_infos

    @write_permitted_user_group_infos.setter
    def write_permitted_user_group_infos(self, write_permitted_user_group_infos):
        """
        Sets the write_permitted_user_group_infos of this ValidationResponse.
        Array of user groups that the user has access to for creating technical support type (`TECH`) support requests.
        Each user group is indicated by its identifier and name (`userGroupId` and `userGroupName`).
        Note: The Customer User Administrator (CUA) can manage user groups by name using
        `My Oracle Cloud Support portal`__.

        __ https://support.oracle.com


        :param write_permitted_user_group_infos: The write_permitted_user_group_infos of this ValidationResponse.
        :type: list[oci.cims.models.CmosUserGroupInfo]
        """
        self._write_permitted_user_group_infos = write_permitted_user_group_infos

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class IncidentResourceType(object):
    """
    Details about the resource associated with the support request.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new IncidentResourceType object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param resource_type_key:
            The value to assign to the resource_type_key property of this IncidentResourceType.
        :type resource_type_key: str

        :param name:
            The value to assign to the name property of this IncidentResourceType.
        :type name: str

        :param label:
            The value to assign to the label property of this IncidentResourceType.
        :type label: str

        :param description:
            The value to assign to the description property of this IncidentResourceType.
        :type description: str

        :param is_subscriptions_supported:
            The value to assign to the is_subscriptions_supported property of this IncidentResourceType.
        :type is_subscriptions_supported: bool

        :param service_category_list:
            The value to assign to the service_category_list property of this IncidentResourceType.
        :type service_category_list: list[oci.cims.models.ServiceCategory]

        :param service:
            The value to assign to the service property of this IncidentResourceType.
        :type service: dict(str, str)

        :param services:
            The value to assign to the services property of this IncidentResourceType.
        :type services: list[oci.cims.models.Services]

        """
        self.swagger_types = {
            'resource_type_key': 'str',
            'name': 'str',
            'label': 'str',
            'description': 'str',
            'is_subscriptions_supported': 'bool',
            'service_category_list': 'list[ServiceCategory]',
            'service': 'dict(str, str)',
            'services': 'list[Services]'
        }
        self.attribute_map = {
            'resource_type_key': 'resourceTypeKey',
            'name': 'name',
            'label': 'label',
            'description': 'description',
            'is_subscriptions_supported': 'isSubscriptionsSupported',
            'service_category_list': 'serviceCategoryList',
            'service': 'service',
            'services': 'services'
        }
        self._resource_type_key = None
        self._name = None
        self._label = None
        self._description = None
        self._is_subscriptions_supported = None
        self._service_category_list = None
        self._service = None
        self._services = None

    @property
    def resource_type_key(self):
        """
        Gets the resource_type_key of this IncidentResourceType.
        A unique identifier for the resource.


        :return: The resource_type_key of this IncidentResourceType.
        :rtype: str
        """
        return self._resource_type_key

    @resource_type_key.setter
    def resource_type_key(self, resource_type_key):
        """
        Sets the resource_type_key of this IncidentResourceType.
        A unique identifier for the resource.


        :param resource_type_key: The resource_type_key of this IncidentResourceType.
        :type: str
        """
        self._resource_type_key = resource_type_key

    @property
    def name(self):
        """
        Gets the name of this IncidentResourceType.
        The display name of the resource.


        :return: The name of this IncidentResourceType.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this IncidentResourceType.
        The display name of the resource.


        :param name: The name of this IncidentResourceType.
        :type: str
        """
        self._name = name

    @property
    def label(self):
        """
        **[Required]** Gets the label of this IncidentResourceType.
        The label associated with the resource.


        :return: The label of this IncidentResourceType.
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """
        Sets the label of this IncidentResourceType.
        The label associated with the resource.


        :param label: The label of this IncidentResourceType.
        :type: str
        """
        self._label = label

    @property
    def description(self):
        """
        Gets the description of this IncidentResourceType.
        The description of the resource.


        :return: The description of this IncidentResourceType.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this IncidentResourceType.
        The description of the resource.


        :param description: The description of this IncidentResourceType.
        :type: str
        """
        self._description = description

    @property
    def is_subscriptions_supported(self):
        """
        Gets the is_subscriptions_supported of this IncidentResourceType.
        Indicates whether multi-subscription is supported


        :return: The is_subscriptions_supported of this IncidentResourceType.
        :rtype: bool
        """
        return self._is_subscriptions_supported

    @is_subscriptions_supported.setter
    def is_subscriptions_supported(self, is_subscriptions_supported):
        """
        Sets the is_subscriptions_supported of this IncidentResourceType.
        Indicates whether multi-subscription is supported


        :param is_subscriptions_supported: The is_subscriptions_supported of this IncidentResourceType.
        :type: bool
        """
        self._is_subscriptions_supported = is_subscriptions_supported

    @property
    def service_category_list(self):
        """
        Gets the service_category_list of this IncidentResourceType.
        The service category list.


        :return: The service_category_list of this IncidentResourceType.
        :rtype: list[oci.cims.models.ServiceCategory]
        """
        return self._service_category_list

    @service_category_list.setter
    def service_category_list(self, service_category_list):
        """
        Sets the service_category_list of this IncidentResourceType.
        The service category list.


        :param service_category_list: The service_category_list of this IncidentResourceType.
        :type: list[oci.cims.models.ServiceCategory]
        """
        self._service_category_list = service_category_list

    @property
    def service(self):
        """
        Gets the service of this IncidentResourceType.
        The map of services for MOS Taxonomy.


        :return: The service of this IncidentResourceType.
        :rtype: dict(str, str)
        """
        return self._service

    @service.setter
    def service(self, service):
        """
        Sets the service of this IncidentResourceType.
        The map of services for MOS Taxonomy.


        :param service: The service of this IncidentResourceType.
        :type: dict(str, str)
        """
        self._service = service

    @property
    def services(self):
        """
        Gets the services of this IncidentResourceType.
        The service categories list for MOS Taxonomy.


        :return: The services of this IncidentResourceType.
        :rtype: list[oci.cims.models.Services]
        """
        return self._services

    @services.setter
    def services(self, services):
        """
        Sets the services of this IncidentResourceType.
        The service categories list for MOS Taxonomy.


        :param services: The services of this IncidentResourceType.
        :type: list[oci.cims.models.Services]
        """
        self._services = services

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class User(object):
    """
    Details about the user.
    """

    #: A constant which can be used with the problem_type property of a User.
    #: This constant has a value of "LIMIT"
    PROBLEM_TYPE_LIMIT = "LIMIT"

    #: A constant which can be used with the problem_type property of a User.
    #: This constant has a value of "LEGACY_LIMIT"
    PROBLEM_TYPE_LEGACY_LIMIT = "LEGACY_LIMIT"

    #: A constant which can be used with the problem_type property of a User.
    #: This constant has a value of "TECH"
    PROBLEM_TYPE_TECH = "TECH"

    #: A constant which can be used with the problem_type property of a User.
    #: This constant has a value of "ACCOUNT"
    PROBLEM_TYPE_ACCOUNT = "ACCOUNT"

    #: A constant which can be used with the problem_type property of a User.
    #: This constant has a value of "TAXONOMY"
    PROBLEM_TYPE_TAXONOMY = "TAXONOMY"

    def __init__(self, **kwargs):
        """
        Initializes a new User object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param key:
            The value to assign to the key property of this User.
        :type key: str

        :param first_name:
            The value to assign to the first_name property of this User.
        :type first_name: str

        :param last_name:
            The value to assign to the last_name property of this User.
        :type last_name: str

        :param country:
            The value to assign to the country property of this User.
        :type country: str

        :param csi:
            The value to assign to the csi property of this User.
        :type csi: str

        :param phone:
            The value to assign to the phone property of this User.
        :type phone: str

        :param timezone:
            The value to assign to the timezone property of this User.
        :type timezone: str

        :param organization_name:
            The value to assign to the organization_name property of this User.
        :type organization_name: str

        :param compartment_id:
            The value to assign to the compartment_id property of this User.
        :type compartment_id: str

        :param contact_email:
            The value to assign to the contact_email property of this User.
        :type contact_email: str

        :param problem_type:
            The value to assign to the problem_type property of this User.
            Allowed values for this property are: "LIMIT", "LEGACY_LIMIT", "TECH", "ACCOUNT", "TAXONOMY"
        :type problem_type: str

        """
        self.swagger_types = {
            'key': 'str',
            'first_name': 'str',
            'last_name': 'str',
            'country': 'str',
            'csi': 'str',
            'phone': 'str',
            'timezone': 'str',
            'organization_name': 'str',
            'compartment_id': 'str',
            'contact_email': 'str',
            'problem_type': 'str'
        }
        self.attribute_map = {
            'key': 'key',
            'first_name': 'firstName',
            'last_name': 'lastName',
            'country': 'country',
            'csi': 'csi',
            'phone': 'phone',
            'timezone': 'timezone',
            'organization_name': 'organizationName',
            'compartment_id': 'compartmentId',
            'contact_email': 'contactEmail',
            'problem_type': 'problemType'
        }
        self._key = None
        self._first_name = None
        self._last_name = None
        self._country = None
        self._csi = None
        self._phone = None
        self._timezone = None
        self._organization_name = None
        self._compartment_id = None
        self._contact_email = None
        self._problem_type = None

    @property
    def key(self):
        """
        **[Required]** Gets the key of this User.
        A unique identifier for the user.


        :return: The key of this User.
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """
        Sets the key of this User.
        A unique identifier for the user.


        :param key: The key of this User.
        :type: str
        """
        self._key = key

    @property
    def first_name(self):
        """
        Gets the first_name of this User.
        The user's first name.


        :return: The first_name of this User.
        :rtype: str
        """
        return self._first_name

    @first_name.setter
    def first_name(self, first_name):
        """
        Sets the first_name of this User.
        The user's first name.


        :param first_name: The first_name of this User.
        :type: str
        """
        self._first_name = first_name

    @property
    def last_name(self):
        """
        Gets the last_name of this User.
        The user's last name.


        :return: The last_name of this User.
        :rtype: str
        """
        return self._last_name

    @last_name.setter
    def last_name(self, last_name):
        """
        Sets the last_name of this User.
        The user's last name.


        :param last_name: The last_name of this User.
        :type: str
        """
        self._last_name = last_name

    @property
    def country(self):
        """
        Gets the country of this User.
        The country of the user.


        :return: The country of this User.
        :rtype: str
        """
        return self._country

    @country.setter
    def country(self, country):
        """
        Sets the country of this User.
        The country of the user.


        :param country: The country of this User.
        :type: str
        """
        self._country = country

    @property
    def csi(self):
        """
        Gets the csi of this User.
        The CSI associated with the user.


        :return: The csi of this User.
        :rtype: str
        """
        return self._csi

    @csi.setter
    def csi(self, csi):
        """
        Sets the csi of this User.
        The CSI associated with the user.


        :param csi: The csi of this User.
        :type: str
        """
        self._csi = csi

    @property
    def phone(self):
        """
        Gets the phone of this User.
        The user's contact phone number.


        :return: The phone of this User.
        :rtype: str
        """
        return self._phone

    @phone.setter
    def phone(self, phone):
        """
        Sets the phone of this User.
        The user's contact phone number.


        :param phone: The phone of this User.
        :type: str
        """
        self._phone = phone

    @property
    def timezone(self):
        """
        Gets the timezone of this User.
        The timezone of the user.


        :return: The timezone of this User.
        :rtype: str
        """
        return self._timezone

    @timezone.setter
    def timezone(self, timezone):
        """
        Sets the timezone of this User.
        The timezone of the user.


        :param timezone: The timezone of this User.
        :type: str
        """
        self._timezone = timezone

    @property
    def organization_name(self):
        """
        Gets the organization_name of this User.
        The company that the user belongs to.


        :return: The organization_name of this User.
        :rtype: str
        """
        return self._organization_name

    @organization_name.setter
    def organization_name(self, organization_name):
        """
        Sets the organization_name of this User.
        The company that the user belongs to.


        :param organization_name: The organization_name of this User.
        :type: str
        """
        self._organization_name = organization_name

    @property
    def compartment_id(self):
        """
        Gets the compartment_id of this User.
        The OCID of the tenancy.


        :return: The compartment_id of this User.
        :rtype: str
        """
        return self._compartment_id

    @compartment_id.setter
    def compartment_id(self, compartment_id):
        """
        Sets the compartment_id of this User.
        The OCID of the tenancy.


        :param compartment_id: The compartment_id of this User.
        :type: str
        """
        self._compartment_id = compartment_id

    @property
    def contact_email(self):
        """
        Gets the contact_email of this User.
        The email of the contact person.


        :return: The contact_email of this User.
        :rtype: str
        """
        return self._contact_email

    @contact_email.setter
    def contact_email(self, contact_email):
        """
        Sets the contact_email of this User.
        The email of the contact person.


        :param contact_email: The contact_email of this User.
        :type: str
        """
        self._contact_email = contact_email

    @property
    def problem_type(self):
        """
        Gets the problem_type of this User.
        The kind of support request (type of support request).
        For information about `ACCOUNT` support requests, see
        `Creating a Billing Support Request`__.
        For information about `LIMIT` support requests, see
        `Creating a Service Limit Increase Request`__.
        For information about `TECH` support requests, see
        `Creating a Technical Support Request`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-billing.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-limit.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-technical.htm

        Allowed values for this property are: "LIMIT", "LEGACY_LIMIT", "TECH", "ACCOUNT", "TAXONOMY"


        :return: The problem_type of this User.
        :rtype: str
        """
        return self._problem_type

    @problem_type.setter
    def problem_type(self, problem_type):
        """
        Sets the problem_type of this User.
        The kind of support request (type of support request).
        For information about `ACCOUNT` support requests, see
        `Creating a Billing Support Request`__.
        For information about `LIMIT` support requests, see
        `Creating a Service Limit Increase Request`__.
        For information about `TECH` support requests, see
        `Creating a Technical Support Request`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-billing.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-limit.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-technical.htm


        :param problem_type: The problem_type of this User.
        :type: str
        """
        allowed_values = ["LIMIT", "LEGACY_LIMIT", "TECH", "ACCOUNT", "TAXONOMY"]
        if not value_allowed_none_or_none_sentinel(problem_type, allowed_values):
            raise ValueError(
                f"Invalid value for `problem_type`, must be None or one of {allowed_values}"
            )
        self._problem_type = problem_type

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231

from .item import Item
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class LimitItem(Item):
    """
    Details about the LimitItem object.
    """

    #: A constant which can be used with the limit_status property of a LimitItem.
    #: This constant has a value of "APPROVED"
    LIMIT_STATUS_APPROVED = "APPROVED"

    #: A constant which can be used with the limit_status property of a LimitItem.
    #: This constant has a value of "PARTIALLY_APPROVED"
    LIMIT_STATUS_PARTIALLY_APPROVED = "PARTIALLY_APPROVED"

    #: A constant which can be used with the limit_status property of a LimitItem.
    #: This constant has a value of "NOT_APPROVED"
    LIMIT_STATUS_NOT_APPROVED = "NOT_APPROVED"

    #: A constant which can be used with the limit_status property of a LimitItem.
    #: This constant has a value of "REJECTED"
    LIMIT_STATUS_REJECTED = "REJECTED"

    def __init__(self, **kwargs):
        """
        Initializes a new LimitItem object with values from keyword arguments. The default value of the :py:attr:`~oci.cims.models.LimitItem.type` attribute
        of this class is ``limit`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param item_key:
            The value to assign to the item_key property of this LimitItem.
        :type item_key: str

        :param name:
            The value to assign to the name property of this LimitItem.
        :type name: str

        :param type:
            The value to assign to the type property of this LimitItem.
        :type type: str

        :param category:
            The value to assign to the category property of this LimitItem.
        :type category: oci.cims.models.Category

        :param sub_category:
            The value to assign to the sub_category property of this LimitItem.
        :type sub_category: oci.cims.models.SubCategory

        :param issue_type:
            The value to assign to the issue_type property of this LimitItem.
        :type issue_type: oci.cims.models.IssueType

        :param current_limit:
            The value to assign to the current_limit property of this LimitItem.
        :type current_limit: int

        :param current_usage:
            The value to assign to the current_usage property of this LimitItem.
        :type current_usage: int

        :param requested_limit:
            The value to assign to the requested_limit property of this LimitItem.
        :type requested_limit: int

        :param limit_status:
            The value to assign to the limit_status property of this LimitItem.
            Allowed values for this property are: "APPROVED", "PARTIALLY_APPROVED", "NOT_APPROVED", "REJECTED", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type limit_status: str

        :param customer_message:
            The value to assign to the customer_message property of this LimitItem.
        :type customer_message: str

        """
        self.swagger_types = {
            'item_key': 'str',
            'name': 'str',
            'type': 'str',
            'category': 'Category',
            'sub_category': 'SubCategory',
            'issue_type': 'IssueType',
            'current_limit': 'int',
            'current_usage': 'int',
            'requested_limit': 'int',
            'limit_status': 'str',
            'customer_message': 'str'
        }
        self.attribute_map = {
            'item_key': 'itemKey',
            'name': 'name',
            'type': 'type',
            'category': 'category',
            'sub_category': 'subCategory',
            'issue_type': 'issueType',
            'current_limit': 'currentLimit',
            'current_usage': 'currentUsage',
            'requested_limit': 'requestedLimit',
            'limit_status': 'limitStatus',
            'customer_message': 'customerMessage'
        }
        self._item_key = None
        self._name = None
        self._type = None
        self._category = None
        self._sub_category = None
        self._issue_type = None
        self._current_limit = None
        self._current_usage = None
        self._requested_limit = None
        self._limit_status = None
        self._customer_message = None
        self._type = 'limit'

    @property
    def current_limit(self):
        """
        Gets the current_limit of this LimitItem.
        The current service limit for the resource.


        :return: The current_limit of this LimitItem.
        :rtype: int
        """
        return self._current_limit

    @current_limit.setter
    def current_limit(self, current_limit):
        """
        Sets the current_limit of this LimitItem.
        The current service limit for the resource.


        :param current_limit: The current_limit of this LimitItem.
        :type: int
        """
        self._current_limit = current_limit

    @property
    def current_usage(self):
        """
        Gets the current_usage of this LimitItem.
        The current resource usage.


        :return: The current_usage of this LimitItem.
        :rtype: int
        """
        return self._current_usage

    @current_usage.setter
    def current_usage(self, current_usage):
        """
        Sets the current_usage of this LimitItem.
        The current resource usage.


        :param current_usage: The current_usage of this LimitItem.
        :type: int
        """
        self._current_usage = current_usage

    @property
    def requested_limit(self):
        """
        Gets the requested_limit of this LimitItem.
        The new service limit being requested for the resource.


        :return: The requested_limit of this LimitItem.
        :rtype: int
        """
        return self._requested_limit

    @requested_limit.setter
    def requested_limit(self, requested_limit):
        """
        Sets the requested_limit of this LimitItem.
        The new service limit being requested for the resource.


        :param requested_limit: The requested_limit of this LimitItem.
        :type: int
        """
        self._requested_limit = requested_limit

    @property
    def limit_status(self):
        """
        Gets the limit_status of this LimitItem.
        The status of the request.

        Allowed values for this property are: "APPROVED", "PARTIALLY_APPROVED", "NOT_APPROVED", "REJECTED", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The limit_status of this LimitItem.
        :rtype: str
        """
        return self._limit_status

    @limit_status.setter
    def limit_status(self, limit_status):
        """
        Sets the limit_status of this LimitItem.
        The status of the request.


        :param limit_status: The limit_status of this LimitItem.
        :type: str
        """
        allowed_values = ["APPROVED", "PARTIALLY_APPROVED", "NOT_APPROVED", "REJECTED"]
        if not value_allowed_none_or_none_sentinel(limit_status, allowed_values):
            limit_status = 'UNKNOWN_ENUM_VALUE'
        self._limit_status = limit_status

    @property
    def customer_message(self):
        """
        Gets the customer_message of this LimitItem.
        The message to customer for partially approved and rejected limit requests


        :return: The customer_message of this LimitItem.
        :rtype: str
        """
        return self._customer_message

    @customer_message.setter
    def customer_message(self, customer_message):
        """
        Sets the customer_message of this LimitItem.
        The message to customer for partially approved and rejected limit requests


        :param customer_message: The customer_message of this LimitItem.
        :type: str
        """
        self._customer_message = customer_message

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class SubComponents(object):
    """
    List of subcomponents under a subcategory.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new SubComponents object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param sub_category:
            The value to assign to the sub_category property of this SubComponents.
        :type sub_category: dict(str, str)

        :param schema:
            The value to assign to the schema property of this SubComponents.
        :type schema: str

        """
        self.swagger_types = {
            'sub_category': 'dict(str, str)',
            'schema': 'str'
        }
        self.attribute_map = {
            'sub_category': 'subCategory',
            'schema': 'schema'
        }
        self._sub_category = None
        self._schema = None

    @property
    def sub_category(self):
        """
        Gets the sub_category of this SubComponents.
        Subcategory list.


        :return: The sub_category of this SubComponents.
        :rtype: dict(str, str)
        """
        return self._sub_category

    @sub_category.setter
    def sub_category(self, sub_category):
        """
        Sets the sub_category of this SubComponents.
        Subcategory list.


        :param sub_category: The sub_category of this SubComponents.
        :type: dict(str, str)
        """
        self._sub_category = sub_category

    @property
    def schema(self):
        """
        Gets the schema of this SubComponents.
        Schema of a subcategory.


        :return: The schema of this SubComponents.
        :rtype: str
        """
        return self._schema

    @schema.setter
    def schema(self, schema):
        """
        Sets the schema of this SubComponents.
        Schema of a subcategory.


        :param schema: The schema of this SubComponents.
        :type: str
        """
        self._schema = schema

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class Incident(object):
    """
    Details about the support request.
    """

    #: A constant which can be used with the problem_type property of a Incident.
    #: This constant has a value of "LIMIT"
    PROBLEM_TYPE_LIMIT = "LIMIT"

    #: A constant which can be used with the problem_type property of a Incident.
    #: This constant has a value of "LEGACY_LIMIT"
    PROBLEM_TYPE_LEGACY_LIMIT = "LEGACY_LIMIT"

    #: A constant which can be used with the problem_type property of a Incident.
    #: This constant has a value of "TECH"
    PROBLEM_TYPE_TECH = "TECH"

    #: A constant which can be used with the problem_type property of a Incident.
    #: This constant has a value of "ACCOUNT"
    PROBLEM_TYPE_ACCOUNT = "ACCOUNT"

    #: A constant which can be used with the problem_type property of a Incident.
    #: This constant has a value of "TAXONOMY"
    PROBLEM_TYPE_TAXONOMY = "TAXONOMY"

    def __init__(self, **kwargs):
        """
        Initializes a new Incident object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param key:
            The value to assign to the key property of this Incident.
        :type key: str

        :param compartment_id:
            The value to assign to the compartment_id property of this Incident.
        :type compartment_id: str

        :param contact_list:
            The value to assign to the contact_list property of this Incident.
        :type contact_list: oci.cims.models.ContactList

        :param tenancy_information:
            The value to assign to the tenancy_information property of this Incident.
        :type tenancy_information: oci.cims.models.TenancyInformation

        :param ticket:
            The value to assign to the ticket property of this Incident.
        :type ticket: oci.cims.models.Ticket

        :param incident_type:
            The value to assign to the incident_type property of this Incident.
        :type incident_type: oci.cims.models.IncidentType

        :param migrated_sr_number:
            The value to assign to the migrated_sr_number property of this Incident.
        :type migrated_sr_number: str

        :param user_group_id:
            The value to assign to the user_group_id property of this Incident.
        :type user_group_id: str

        :param user_group_name:
            The value to assign to the user_group_name property of this Incident.
        :type user_group_name: str

        :param primary_contact_party_id:
            The value to assign to the primary_contact_party_id property of this Incident.
        :type primary_contact_party_id: str

        :param primary_contact_party_name:
            The value to assign to the primary_contact_party_name property of this Incident.
        :type primary_contact_party_name: str

        :param is_write_permitted:
            The value to assign to the is_write_permitted property of this Incident.
        :type is_write_permitted: bool

        :param warn_message:
            The value to assign to the warn_message property of this Incident.
        :type warn_message: str

        :param problem_type:
            The value to assign to the problem_type property of this Incident.
            Allowed values for this property are: "LIMIT", "LEGACY_LIMIT", "TECH", "ACCOUNT", "TAXONOMY", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type problem_type: str

        :param referrer:
            The value to assign to the referrer property of this Incident.
        :type referrer: str

        """
        self.swagger_types = {
            'key': 'str',
            'compartment_id': 'str',
            'contact_list': 'ContactList',
            'tenancy_information': 'TenancyInformation',
            'ticket': 'Ticket',
            'incident_type': 'IncidentType',
            'migrated_sr_number': 'str',
            'user_group_id': 'str',
            'user_group_name': 'str',
            'primary_contact_party_id': 'str',
            'primary_contact_party_name': 'str',
            'is_write_permitted': 'bool',
            'warn_message': 'str',
            'problem_type': 'str',
            'referrer': 'str'
        }
        self.attribute_map = {
            'key': 'key',
            'compartment_id': 'compartmentId',
            'contact_list': 'contactList',
            'tenancy_information': 'tenancyInformation',
            'ticket': 'ticket',
            'incident_type': 'incidentType',
            'migrated_sr_number': 'migratedSrNumber',
            'user_group_id': 'userGroupId',
            'user_group_name': 'userGroupName',
            'primary_contact_party_id': 'primaryContactPartyId',
            'primary_contact_party_name': 'primaryContactPartyName',
            'is_write_permitted': 'isWritePermitted',
            'warn_message': 'warnMessage',
            'problem_type': 'problemType',
            'referrer': 'referrer'
        }
        self._key = None
        self._compartment_id = None
        self._contact_list = None
        self._tenancy_information = None
        self._ticket = None
        self._incident_type = None
        self._migrated_sr_number = None
        self._user_group_id = None
        self._user_group_name = None
        self._primary_contact_party_id = None
        self._primary_contact_party_name = None
        self._is_write_permitted = None
        self._warn_message = None
        self._problem_type = None
        self._referrer = None

    @property
    def key(self):
        """
        **[Required]** Gets the key of this Incident.
        Unique identifier for the support request.


        :return: The key of this Incident.
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """
        Sets the key of this Incident.
        Unique identifier for the support request.


        :param key: The key of this Incident.
        :type: str
        """
        self._key = key

    @property
    def compartment_id(self):
        """
        Gets the compartment_id of this Incident.
        The OCID of the tenancy.


        :return: The compartment_id of this Incident.
        :rtype: str
        """
        return self._compartment_id

    @compartment_id.setter
    def compartment_id(self, compartment_id):
        """
        Sets the compartment_id of this Incident.
        The OCID of the tenancy.


        :param compartment_id: The compartment_id of this Incident.
        :type: str
        """
        self._compartment_id = compartment_id

    @property
    def contact_list(self):
        """
        Gets the contact_list of this Incident.

        :return: The contact_list of this Incident.
        :rtype: oci.cims.models.ContactList
        """
        return self._contact_list

    @contact_list.setter
    def contact_list(self, contact_list):
        """
        Sets the contact_list of this Incident.

        :param contact_list: The contact_list of this Incident.
        :type: oci.cims.models.ContactList
        """
        self._contact_list = contact_list

    @property
    def tenancy_information(self):
        """
        Gets the tenancy_information of this Incident.

        :return: The tenancy_information of this Incident.
        :rtype: oci.cims.models.TenancyInformation
        """
        return self._tenancy_information

    @tenancy_information.setter
    def tenancy_information(self, tenancy_information):
        """
        Sets the tenancy_information of this Incident.

        :param tenancy_information: The tenancy_information of this Incident.
        :type: oci.cims.models.TenancyInformation
        """
        self._tenancy_information = tenancy_information

    @property
    def ticket(self):
        """
        Gets the ticket of this Incident.

        :return: The ticket of this Incident.
        :rtype: oci.cims.models.Ticket
        """
        return self._ticket

    @ticket.setter
    def ticket(self, ticket):
        """
        Sets the ticket of this Incident.

        :param ticket: The ticket of this Incident.
        :type: oci.cims.models.Ticket
        """
        self._ticket = ticket

    @property
    def incident_type(self):
        """
        Gets the incident_type of this Incident.

        :return: The incident_type of this Incident.
        :rtype: oci.cims.models.IncidentType
        """
        return self._incident_type

    @incident_type.setter
    def incident_type(self, incident_type):
        """
        Sets the incident_type of this Incident.

        :param incident_type: The incident_type of this Incident.
        :type: oci.cims.models.IncidentType
        """
        self._incident_type = incident_type

    @property
    def migrated_sr_number(self):
        """
        Gets the migrated_sr_number of this Incident.
        Technical support type (`TECH`) only: The pre-migration identifier of the support request in My Oracle Support (MOS).


        :return: The migrated_sr_number of this Incident.
        :rtype: str
        """
        return self._migrated_sr_number

    @migrated_sr_number.setter
    def migrated_sr_number(self, migrated_sr_number):
        """
        Sets the migrated_sr_number of this Incident.
        Technical support type (`TECH`) only: The pre-migration identifier of the support request in My Oracle Support (MOS).


        :param migrated_sr_number: The migrated_sr_number of this Incident.
        :type: str
        """
        self._migrated_sr_number = migrated_sr_number

    @property
    def user_group_id(self):
        """
        Gets the user_group_id of this Incident.
        Technical support type (`TECH`) only: The identifier of the support request's user group in My Oracle Cloud Support portal.


        :return: The user_group_id of this Incident.
        :rtype: str
        """
        return self._user_group_id

    @user_group_id.setter
    def user_group_id(self, user_group_id):
        """
        Sets the user_group_id of this Incident.
        Technical support type (`TECH`) only: The identifier of the support request's user group in My Oracle Cloud Support portal.


        :param user_group_id: The user_group_id of this Incident.
        :type: str
        """
        self._user_group_id = user_group_id

    @property
    def user_group_name(self):
        """
        Gets the user_group_name of this Incident.
        Technical support type (`TECH`) only: Name of the support request's user group in My Oracle Cloud Support portal.


        :return: The user_group_name of this Incident.
        :rtype: str
        """
        return self._user_group_name

    @user_group_name.setter
    def user_group_name(self, user_group_name):
        """
        Sets the user_group_name of this Incident.
        Technical support type (`TECH`) only: Name of the support request's user group in My Oracle Cloud Support portal.


        :param user_group_name: The user_group_name of this Incident.
        :type: str
        """
        self._user_group_name = user_group_name

    @property
    def primary_contact_party_id(self):
        """
        Gets the primary_contact_party_id of this Incident.
        Technical support type (`TECH`) only: The identifier of the support request's primary contact (`primaryContactPartyName`) in My Oracle Cloud Support portal.


        :return: The primary_contact_party_id of this Incident.
        :rtype: str
        """
        return self._primary_contact_party_id

    @primary_contact_party_id.setter
    def primary_contact_party_id(self, primary_contact_party_id):
        """
        Sets the primary_contact_party_id of this Incident.
        Technical support type (`TECH`) only: The identifier of the support request's primary contact (`primaryContactPartyName`) in My Oracle Cloud Support portal.


        :param primary_contact_party_id: The primary_contact_party_id of this Incident.
        :type: str
        """
        self._primary_contact_party_id = primary_contact_party_id

    @property
    def primary_contact_party_name(self):
        """
        Gets the primary_contact_party_name of this Incident.
        Technical support type (`TECH`) only: The name of the support request's primary contact in My Oracle Cloud Support portal.


        :return: The primary_contact_party_name of this Incident.
        :rtype: str
        """
        return self._primary_contact_party_name

    @primary_contact_party_name.setter
    def primary_contact_party_name(self, primary_contact_party_name):
        """
        Sets the primary_contact_party_name of this Incident.
        Technical support type (`TECH`) only: The name of the support request's primary contact in My Oracle Cloud Support portal.


        :param primary_contact_party_name: The primary_contact_party_name of this Incident.
        :type: str
        """
        self._primary_contact_party_name = primary_contact_party_name

    @property
    def is_write_permitted(self):
        """
        Gets the is_write_permitted of this Incident.
        Technical support type (`TECH`) only: Allows update of the support request in My Oracle Cloud Support portal,
        when the user has write permission to the support request's user group.


        :return: The is_write_permitted of this Incident.
        :rtype: bool
        """
        return self._is_write_permitted

    @is_write_permitted.setter
    def is_write_permitted(self, is_write_permitted):
        """
        Sets the is_write_permitted of this Incident.
        Technical support type (`TECH`) only: Allows update of the support request in My Oracle Cloud Support portal,
        when the user has write permission to the support request's user group.


        :param is_write_permitted: The is_write_permitted of this Incident.
        :type: bool
        """
        self._is_write_permitted = is_write_permitted

    @property
    def warn_message(self):
        """
        Gets the warn_message of this Incident.
        Technical support type (`TECH`) only: Message indicating the user group (`userGroupId`) that was auto-selected for a new support request. This message appears when no user group was specified in the create request for a new technical support request.


        :return: The warn_message of this Incident.
        :rtype: str
        """
        return self._warn_message

    @warn_message.setter
    def warn_message(self, warn_message):
        """
        Sets the warn_message of this Incident.
        Technical support type (`TECH`) only: Message indicating the user group (`userGroupId`) that was auto-selected for a new support request. This message appears when no user group was specified in the create request for a new technical support request.


        :param warn_message: The warn_message of this Incident.
        :type: str
        """
        self._warn_message = warn_message

    @property
    def problem_type(self):
        """
        Gets the problem_type of this Incident.
        The kind of support request (type of support request).
        For information about `ACCOUNT` support requests, see
        `Creating a Billing Support Request`__.
        For information about `LIMIT` support requests, see
        `Creating a Service Limit Increase Request`__.
        For information about `TECH` support requests, see
        `Creating a Technical Support Request`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-billing.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-limit.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-technical.htm

        Allowed values for this property are: "LIMIT", "LEGACY_LIMIT", "TECH", "ACCOUNT", "TAXONOMY", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The problem_type of this Incident.
        :rtype: str
        """
        return self._problem_type

    @problem_type.setter
    def problem_type(self, problem_type):
        """
        Sets the problem_type of this Incident.
        The kind of support request (type of support request).
        For information about `ACCOUNT` support requests, see
        `Creating a Billing Support Request`__.
        For information about `LIMIT` support requests, see
        `Creating a Service Limit Increase Request`__.
        For information about `TECH` support requests, see
        `Creating a Technical Support Request`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-billing.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-limit.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident-technical.htm


        :param problem_type: The problem_type of this Incident.
        :type: str
        """
        allowed_values = ["LIMIT", "LEGACY_LIMIT", "TECH", "ACCOUNT", "TAXONOMY"]
        if not value_allowed_none_or_none_sentinel(problem_type, allowed_values):
            problem_type = 'UNKNOWN_ENUM_VALUE'
        self._problem_type = problem_type

    @property
    def referrer(self):
        """
        Gets the referrer of this Incident.
        The incident referrer. This value is often the URL that the customer used when creating the support request.


        :return: The referrer of this Incident.
        :rtype: str
        """
        return self._referrer

    @referrer.setter
    def referrer(self, referrer):
        """
        Sets the referrer of this Incident.
        The incident referrer. This value is often the URL that the customer used when creating the support request.


        :param referrer: The referrer of this Incident.
        :type: str
        """
        self._referrer = referrer

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231

from __future__ import absolute_import

from oci._vendor import requests  # noqa: F401
from oci._vendor import six

from oci import retry, circuit_breaker  # noqa: F401
from oci.base_client import BaseClient
from oci.config import get_config_value_or_default, validate_config
from oci.signer import Signer
from oci.util import Sentinel, get_signer_from_authentication_type, AUTHENTICATION_TYPE_FIELD_NAME
from oci.util import back_up_body_calculate_stream_content_length, is_content_length_calculable_by_req_util
from oci.exceptions import InvalidDeveloperToolConfiguration
from oci.developer_tool_configuration import OCI_SDK_ENABLED_SERVICES_SET
from .models import cims_type_mapping
missing = Sentinel("Missing")


class IncidentClient(object):
    """
    Use the Support Management API to manage support requests.
    For more information, see [Getting Help and Contacting Support](/iaas/Content/GSG/Tasks/contactingsupport.htm).
    **Note**: Before you can create service requests with this API,
    complete user registration at My Oracle Cloud Support
    and then ask your tenancy administrator to provide you authorization for the related user groups.
    """

    def __init__(self, config, **kwargs):
        """
        Creates a new service client

        :param dict config:
            Configuration keys and values as per `SDK and Tool Configuration <https://docs.oracle.com/iaas/Content/API/Concepts/sdkconfig.htm>`__.
            The :py:meth:`~oci.config.from_file` method can be used to load configuration from a file. Alternatively, a ``dict`` can be passed. You can validate_config
            the dict using :py:meth:`~oci.config.validate_config`

        :param str service_endpoint: (optional)
            The endpoint of the service to call using this client. For example ``https://iaas.us-ashburn-1.oraclecloud.com``. If this keyword argument is
            not provided then it will be derived using the region in the config parameter. You should only provide this keyword argument if you have an explicit
            need to specify a service endpoint.

        :param timeout: (optional)
            The connection and read timeouts for the client. The default values are connection timeout 10 seconds and read timeout 60 seconds. This keyword argument can be provided
            as a single float, in which case the value provided is used for both the read and connection timeouts, or as a tuple of two floats. If
            a tuple is provided then the first value is used as the connection timeout and the second value as the read timeout.
        :type timeout: float or tuple(float, float)

        :param signer: (optional)
            The signer to use when signing requests made by the service client. The default is to use a :py:class:`~oci.signer.Signer` based on the values
            provided in the config parameter.

            One use case for this parameter is for `Instance Principals authentication <https://docs.oracle.com/iaas/Content/Identity/Tasks/callingservicesfrominstances.htm>`__
            by passing an instance of :py:class:`~oci.auth.signers.InstancePrincipalsSecurityTokenSigner` as the value for this keyword argument
        :type signer: :py:class:`~oci.signer.AbstractBaseSigner`

        :param obj retry_strategy: (optional)
            A retry strategy to apply to all calls made by this service client (i.e. at the client level). There is no retry strategy applied by default.
            Retry strategies can also be applied at the operation level by passing a ``retry_strategy`` keyword argument as part of calling the operation.
            Any value provided at the operation level will override whatever is specified at the client level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. A convenience :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY`
            is also available. The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

        :param obj circuit_breaker_strategy: (optional)
            A circuit breaker strategy to apply to all calls made by this service client (i.e. at the client level).
            This client uses :py:data:`~oci.circuit_breaker.DEFAULT_CIRCUIT_BREAKER_STRATEGY` as default if no circuit breaker strategy is provided.
            The specifics of circuit breaker strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/circuit_breakers.html>`__.

        :param function circuit_breaker_callback: (optional)
            Callback function to receive any exceptions triggerred by the circuit breaker.

        :param bool client_level_realm_specific_endpoint_template_enabled: (optional)
            A boolean flag to indicate whether or not this client should be created with realm specific endpoint template enabled or disable. By default, this will be set as None.

        :param allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this client should allow control characters in the response object. By default, the client will not
            allow control characters to be in the response object.

        :param enable_strict_url_encoding: (optional)
            enable_strict_url_encoding is a boolean to indicate whether or not this client should enable strict url encoding in path params of a request.
            By default, the client will not enable strict url encoding
        """
        if not OCI_SDK_ENABLED_SERVICES_SET.is_service_enabled("cims"):
            raise InvalidDeveloperToolConfiguration("The Developer Tool Configuration has disabled this service, this behavior is controlled by OCI_SDK_ENABLED_SERVICES_SET variable. Please check if your local developer-tool-configuration file configured the service you're targeting or contact the cloud provider on the availability of this service")

        validate_config(config, signer=kwargs.get('signer'))
        if 'signer' in kwargs:
            signer = kwargs['signer']

        elif AUTHENTICATION_TYPE_FIELD_NAME in config:
            signer = get_signer_from_authentication_type(config)

        else:
            signer = Signer(
                tenancy=config["tenancy"],
                user=config["user"],
                fingerprint=config["fingerprint"],
                private_key_file_location=config.get("key_file"),
                pass_phrase=get_config_value_or_default(config, "pass_phrase"),
                private_key_content=config.get("key_content")
            )

        base_client_init_kwargs = {
            'regional_client': True,
            'service_endpoint': kwargs.get('service_endpoint'),
            'base_path': '/20181231',
            'service_endpoint_template': 'https://incidentmanagement.{region}.oci.{secondLevelDomain}',
            'service_endpoint_template_per_realm': {  },  # noqa: E201 E202
            'service_uses_dualstack_endpoints_by_default': False,
            'skip_deserialization': kwargs.get('skip_deserialization', False),
            'circuit_breaker_strategy': kwargs.get('circuit_breaker_strategy', circuit_breaker.GLOBAL_CIRCUIT_BREAKER_STRATEGY),
            'client_level_realm_specific_endpoint_template_enabled': kwargs.get('client_level_realm_specific_endpoint_template_enabled'),
            'client_level_dualstack_endpoints_enabled': kwargs.get('client_level_dualstack_endpoints_enabled')
        }
        if 'timeout' in kwargs:
            base_client_init_kwargs['timeout'] = kwargs.get('timeout')
        if base_client_init_kwargs.get('circuit_breaker_strategy') is None:
            base_client_init_kwargs['circuit_breaker_strategy'] = circuit_breaker.DEFAULT_CIRCUIT_BREAKER_STRATEGY
        if 'allow_control_chars' in kwargs:
            base_client_init_kwargs['allow_control_chars'] = kwargs.get('allow_control_chars')
        if 'enable_strict_url_encoding' in kwargs:
            base_client_init_kwargs['enable_strict_url_encoding'] = kwargs.get('enable_strict_url_encoding')
        self.base_client = BaseClient("incident", config, signer, cims_type_mapping, **base_client_init_kwargs)
        self.retry_strategy = kwargs.get('retry_strategy')
        self.circuit_breaker_callback = kwargs.get('circuit_breaker_callback')

    def create_incident(self, create_incident_details, **kwargs):
        """
        Creates a support request in the specified tenancy.
        For more information, see `Creating Support Requests`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/create-incident.htm


        :param oci.cims.models.CreateIncident create_incident_details: (required)
            Incident information

        :param str opc_request_id: (optional)
            Unique Oracle-assigned identifier for the request. If you need to contact Oracle about a particular request, please provide the request ID.

        :param str ocid: (optional)
            User OCID for Oracle Identity Cloud Service (IDCS) users who also have a federated Oracle Cloud Infrastructure account.
            User OCID is mandatory for OCI Users and optional for Multicloud users.

        :param str homeregion: (optional)
            The region of the tenancy.

        :param str bearertokentype: (optional)
            Token type that determine which cloud provider the request come from.

        :param str bearertoken: (optional)
            Token that provided by multi cloud provider, which help to validate the email.

        :param str idtoken: (optional)
            IdToken that provided by multi cloud provider, which help to validate the email.

        :param str domainid: (optional)
            The OCID of identity domain.
            DomainID is mandatory if the user is part of Non Default Identity domain.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :param bool enable_strict_url_encoding: (optional)
            enable_strict_url_encoding is a boolean to indicate whether or not this request should enable strict url encoding for path params.
            By default, strict url encoding for path params is disabled

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.cims.models.Incident`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.oracle.com/en-us/iaas/tools/python-sdk-examples/latest/cims/create_incident.py.html>`__ to see an example of how to use create_incident API.
        """
        # Required path and query arguments. These are in camelCase to replace values in service endpoints.
        required_arguments = []
        resource_path = "/v2/incidents"
        method = "POST"
        operation_name = "create_incident"
        api_reference_link = "https://docs.oracle.com/iaas/api/#/en/incidentmanagement/20181231/Incident/CreateIncident"

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "enable_strict_url_encoding",
            "retry_strategy",
            "opc_request_id",
            "ocid",
            "homeregion",
            "bearertokentype",
            "bearertoken",
            "idtoken",
            "domainid"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                f"create_incident got unknown kwargs: {extra_kwargs!r}")

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "opc-request-id": kwargs.get("opc_request_id", missing),
            "ocid": kwargs.get("ocid", missing),
            "homeregion": kwargs.get("homeregion", missing),
            "bearertokentype": kwargs.get("bearertokentype", missing),
            "bearertoken": kwargs.get("bearertoken", missing),
            "idtoken": kwargs.get("idtoken", missing),
            "domainid": kwargs.get("domainid", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                header_params=header_params,
                body=create_incident_details,
                response_type="Incident",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                header_params=header_params,
                body=create_incident_details,
                response_type="Incident",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)

    def get_incident(self, incident_key, compartment_id, **kwargs):
        """
        Gets the specified support request.
        For more information, see `Getting Details for a Support Request`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/get-incident.htm


        :param str incident_key: (required)
            Unique identifier for the support request.

        :param str compartment_id: (required)
            The OCID of the tenancy.

        :param str opc_request_id: (optional)
            Unique Oracle-assigned identifier for the request. If you need to contact Oracle about a particular request, please provide the request ID.

        :param str csi: (optional)
            The Customer Support Identifier (CSI) number associated with the support account.
            The CSI is optional for all support request types.

        :param str ocid: (optional)
            User OCID for Oracle Identity Cloud Service (IDCS) users who also have a federated Oracle Cloud Infrastructure account.
            User OCID is mandatory for OCI Users and optional for Multicloud users.

        :param str homeregion: (optional)
            The region of the tenancy.

        :param str problemtype: (optional)
            The kind of support request.

        :param str bearertokentype: (optional)
            Token type that determine which cloud provider the request come from.

        :param str bearertoken: (optional)
            Token that provided by multi cloud provider, which help to validate the email.

        :param str idtoken: (optional)
            IdToken that provided by multi cloud provider, which help to validate the email.

        :param str domainid: (optional)
            The OCID of identity domain.
            DomainID is mandatory if the user is part of Non Default Identity domain.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :param bool enable_strict_url_encoding: (optional)
            enable_strict_url_encoding is a boolean to indicate whether or not this request should enable strict url encoding for path params.
            By default, strict url encoding for path params is disabled

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.cims.models.Incident`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.oracle.com/en-us/iaas/tools/python-sdk-examples/latest/cims/get_incident.py.html>`__ to see an example of how to use get_incident API.
        """
        # Required path and query arguments. These are in camelCase to replace values in service endpoints.
        required_arguments = ['incidentKey', 'compartmentId']
        resource_path = "/v2/incidents/{incidentKey}"
        method = "GET"
        operation_name = "get_incident"
        api_reference_link = "https://docs.oracle.com/iaas/api/#/en/incidentmanagement/20181231/Incident/GetIncident"

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "enable_strict_url_encoding",
            "retry_strategy",
            "opc_request_id",
            "csi",
            "ocid",
            "homeregion",
            "problemtype",
            "bearertokentype",
            "bearertoken",
            "idtoken",
            "domainid"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                f"get_incident got unknown kwargs: {extra_kwargs!r}")

        path_params = {
            "incidentKey": incident_key
        }

        path_params = {k: v for (k, v) in six.iteritems(path_params) if v is not missing}

        for (k, v) in six.iteritems(path_params):
            if v is None or (isinstance(v, six.string_types) and len(v.strip()) == 0):
                raise ValueError(f'Parameter {k} cannot be None, whitespace or empty string')

        query_params = {
            "compartmentId": compartment_id
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "opc-request-id": kwargs.get("opc_request_id", missing),
            "csi": kwargs.get("csi", missing),
            "ocid": kwargs.get("ocid", missing),
            "homeregion": kwargs.get("homeregion", missing),
            "problemtype": kwargs.get("problemtype", missing),
            "bearertokentype": kwargs.get("bearertokentype", missing),
            "bearertoken": kwargs.get("bearertoken", missing),
            "idtoken": kwargs.get("idtoken", missing),
            "domainid": kwargs.get("domainid", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                response_type="Incident",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                response_type="Incident",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)

    def list_incident_resource_types(self, problem_type, compartment_id, **kwargs):
        """
        Depending on the selected `productType`, either
        lists available products (service groups, services, service categories, and subcategories) for technical support requests or
        lists limits and current usage for limit increase tickets.
        This operation is called during creation of technical support and limit increase tickets.
        For more information about listing products, see
        `Listing Products for Support Requests`__.
        For more information about listing limits, see
        `Listing Limits for Service Limit Increase Requests`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/list-incident-resource-types-taxonomy.htm
        __ https://docs.oracle.com/iaas/Content/GSG/support/list-incident-resource-types-limit.htm


        :param str problem_type: (required)
            The kind of support request.

        :param str compartment_id: (required)
            The OCID of the tenancy.

        :param str opc_request_id: (optional)
            Unique Oracle-assigned identifier for the request. If you need to contact Oracle about a particular request, please provide the request ID.

        :param int limit: (optional)
            For list pagination. The maximum number of results per page, or items to return in a paginated \"List\" call. For important details about how pagination works, see `List Pagination`__.

            __ https://docs.oracle.com/iaas/Content/API/Concepts/usingapi.htm#nine

        :param str page: (optional)
            For list pagination. The value of the `opc-next-page` response header from the previous \"List\" call. For important details about how pagination works, see `List Pagination`__.

            __ https://docs.oracle.com/iaas/Content/API/Concepts/usingapi.htm#nine

        :param str sort_by: (optional)
            The key to use to sort the returned items.

            Allowed values are: "dateUpdated", "severity"

        :param str sort_order: (optional)
            The order to sort the results in.

            Allowed values are: "ASC", "DESC"

        :param str name: (optional)
            The user-friendly name of the support request type.

        :param str csi: (optional)
            The Customer Support Identifier (CSI) number associated with the support account.
            The CSI is optional for all support request types.

        :param str ocid: (optional)
            User OCID for Oracle Identity Cloud Service (IDCS) users who also have a federated Oracle Cloud Infrastructure account.
            User OCID is mandatory for OCI Users and optional for Multicloud users.

        :param str homeregion: (optional)
            The region of the tenancy.

        :param str domainid: (optional)
            The OCID of identity domain.
            DomainID is mandatory if the user is part of Non Default Identity domain.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :param bool enable_strict_url_encoding: (optional)
            enable_strict_url_encoding is a boolean to indicate whether or not this request should enable strict url encoding for path params.
            By default, strict url encoding for path params is disabled

        :return: A :class:`~oci.response.Response` object with data of type list of :class:`~oci.cims.models.IncidentResourceType`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.oracle.com/en-us/iaas/tools/python-sdk-examples/latest/cims/list_incident_resource_types.py.html>`__ to see an example of how to use list_incident_resource_types API.
        """
        # Required path and query arguments. These are in camelCase to replace values in service endpoints.
        required_arguments = ['problemType', 'compartmentId']
        resource_path = "/v2/incidents/incidentResourceTypes"
        method = "GET"
        operation_name = "list_incident_resource_types"
        api_reference_link = "https://docs.oracle.com/iaas/api/#/en/incidentmanagement/20181231/IncidentResourceType/ListIncidentResourceTypes"

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "enable_strict_url_encoding",
            "retry_strategy",
            "opc_request_id",
            "limit",
            "page",
            "sort_by",
            "sort_order",
            "name",
            "csi",
            "ocid",
            "homeregion",
            "domainid"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                f"list_incident_resource_types got unknown kwargs: {extra_kwargs!r}")

        if 'sort_by' in kwargs:
            sort_by_allowed_values = ["dateUpdated", "severity"]
            if kwargs['sort_by'] not in sort_by_allowed_values:
                raise ValueError(
                    f"Invalid value for `sort_by`, must be one of { sort_by_allowed_values }"
                )

        if 'sort_order' in kwargs:
            sort_order_allowed_values = ["ASC", "DESC"]
            if kwargs['sort_order'] not in sort_order_allowed_values:
                raise ValueError(
                    f"Invalid value for `sort_order`, must be one of { sort_order_allowed_values }"
                )

        query_params = {
            "problemType": problem_type,
            "limit": kwargs.get("limit", missing),
            "page": kwargs.get("page", missing),
            "sortBy": kwargs.get("sort_by", missing),
            "sortOrder": kwargs.get("sort_order", missing),
            "compartmentId": compartment_id,
            "name": kwargs.get("name", missing)
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "opc-request-id": kwargs.get("opc_request_id", missing),
            "csi": kwargs.get("csi", missing),
            "ocid": kwargs.get("ocid", missing),
            "homeregion": kwargs.get("homeregion", missing),
            "domainid": kwargs.get("domainid", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                query_params=query_params,
                header_params=header_params,
                response_type="list[IncidentResourceType]",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                query_params=query_params,
                header_params=header_params,
                response_type="list[IncidentResourceType]",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)

    def list_incidents(self, compartment_id, **kwargs):
        """
        Lists support requests for the specified tenancy.
        For more information, see `Listing Support Requests`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/list-incidents.htm


        :param str compartment_id: (required)
            The OCID of the tenancy.

        :param str csi: (optional)
            The Customer Support Identifier (CSI) number associated with the support account.
            The CSI is optional for all support request types.

        :param int limit: (optional)
            For list pagination. The maximum number of results per page, or items to return in a paginated \"List\" call. For important details about how pagination works, see `List Pagination`__.

            __ https://docs.oracle.com/iaas/Content/API/Concepts/usingapi.htm#nine

        :param str sort_by: (optional)
            The key to use to sort the returned items.

            Allowed values are: "dateUpdated", "severity"

        :param str sort_order: (optional)
            The order to sort the results in.

            Allowed values are: "ASC", "DESC"

        :param str lifecycle_state: (optional)
            The current state of the ticket.

            Allowed values are: "ACTIVE", "CLOSED"

        :param str page: (optional)
            For list pagination. The value of the `opc-next-page` response header from the previous \"List\" call. For important details about how pagination works, see `List Pagination`__.

            __ https://docs.oracle.com/iaas/Content/API/Concepts/usingapi.htm#nine

        :param str opc_request_id: (optional)
            Unique Oracle-assigned identifier for the request. If you need to contact Oracle about a particular request, please provide the request ID.

        :param str ocid: (optional)
            User OCID for Oracle Identity Cloud Service (IDCS) users who also have a federated Oracle Cloud Infrastructure account.
            User OCID is mandatory for OCI Users and optional for Multicloud users.

        :param str homeregion: (optional)
            The region of the tenancy.

        :param str problem_type: (optional)
            The kind of support request.

        :param str bearertokentype: (optional)
            Token type that determine which cloud provider the request come from.

        :param str bearertoken: (optional)
            Token that provided by multi cloud provider, which help to validate the email.

        :param str idtoken: (optional)
            IdToken that provided by multi cloud provider, which help to validate the email.

        :param str domainid: (optional)
            The OCID of identity domain.
            DomainID is mandatory if the user is part of Non Default Identity domain.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :param bool enable_strict_url_encoding: (optional)
            enable_strict_url_encoding is a boolean to indicate whether or not this request should enable strict url encoding for path params.
            By default, strict url encoding for path params is disabled

        :return: A :class:`~oci.response.Response` object with data of type list of :class:`~oci.cims.models.IncidentSummary`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.oracle.com/en-us/iaas/tools/python-sdk-examples/latest/cims/list_incidents.py.html>`__ to see an example of how to use list_incidents API.
        """
        # Required path and query arguments. These are in camelCase to replace values in service endpoints.
        required_arguments = ['compartmentId']
        resource_path = "/v2/incidents"
        method = "GET"
        operation_name = "list_incidents"
        api_reference_link = "https://docs.oracle.com/iaas/api/#/en/incidentmanagement/20181231/IncidentSummary/ListIncidents"

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "enable_strict_url_encoding",
            "retry_strategy",
            "csi",
            "limit",
            "sort_by",
            "sort_order",
            "lifecycle_state",
            "page",
            "opc_request_id",
            "ocid",
            "homeregion",
            "problem_type",
            "bearertokentype",
            "bearertoken",
            "idtoken",
            "domainid"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                f"list_incidents got unknown kwargs: {extra_kwargs!r}")

        if 'sort_by' in kwargs:
            sort_by_allowed_values = ["dateUpdated", "severity"]
            if kwargs['sort_by'] not in sort_by_allowed_values:
                raise ValueError(
                    f"Invalid value for `sort_by`, must be one of { sort_by_allowed_values }"
                )

        if 'sort_order' in kwargs:
            sort_order_allowed_values = ["ASC", "DESC"]
            if kwargs['sort_order'] not in sort_order_allowed_values:
                raise ValueError(
                    f"Invalid value for `sort_order`, must be one of { sort_order_allowed_values }"
                )

        if 'lifecycle_state' in kwargs:
            lifecycle_state_allowed_values = ["ACTIVE", "CLOSED"]
            if kwargs['lifecycle_state'] not in lifecycle_state_allowed_values:
                raise ValueError(
                    f"Invalid value for `lifecycle_state`, must be one of { lifecycle_state_allowed_values }"
                )

        query_params = {
            "limit": kwargs.get("limit", missing),
            "compartmentId": compartment_id,
            "sortBy": kwargs.get("sort_by", missing),
            "sortOrder": kwargs.get("sort_order", missing),
            "lifecycleState": kwargs.get("lifecycle_state", missing),
            "page": kwargs.get("page", missing),
            "problemType": kwargs.get("problem_type", missing)
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "csi": kwargs.get("csi", missing),
            "opc-request-id": kwargs.get("opc_request_id", missing),
            "ocid": kwargs.get("ocid", missing),
            "homeregion": kwargs.get("homeregion", missing),
            "bearertokentype": kwargs.get("bearertokentype", missing),
            "bearertoken": kwargs.get("bearertoken", missing),
            "idtoken": kwargs.get("idtoken", missing),
            "domainid": kwargs.get("domainid", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                query_params=query_params,
                header_params=header_params,
                response_type="list[IncidentSummary]",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                query_params=query_params,
                header_params=header_params,
                response_type="list[IncidentSummary]",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)

    def put_attachment(self, put_attachment_details, incident_key, attachment_name, compartment_id, is_restricted_flag, **kwargs):
        """
        Uploads the file and attaches it to the support request.


        :param stream put_attachment_details: (required)
            File to be uploaded as attachment to the Service Request.

        :param str incident_key: (required)
            Unique identifier for the support request.

        :param str attachment_name: (required)
            The name of the file to attach to the support request. Avoid entering confidential information.

        :param str compartment_id: (required)
            The OCID of the tenancy.

        :param bool is_restricted_flag: (required)
            Set to `true` when the attachment contains personal information (PI)
            or protected health information (PHI).

        :param str csi: (optional)
            The Customer Support Identifier (CSI) number associated with the support account.
            The CSI is optional for all support request types.

        :param str opc_request_id: (optional)
            Unique Oracle-assigned identifier for the request. If you need to contact Oracle about a particular request, please provide the request ID.

        :param str ocid: (optional)
            User OCID for Oracle Identity Cloud Service (IDCS) users who also have a federated Oracle Cloud Infrastructure account.
            User OCID is mandatory for OCI Users and optional for Multicloud users.

        :param str if_match: (optional)
            For optimistic concurrency control. In the PUT or DELETE call for a resource, set the `if-match` parameter to the value of the etag from a previous GET or POST response for that resource. The resource will be updated or deleted only if the etag you provide matches the resource's current etag value.

        :param str homeregion: (optional)
            The region of the tenancy.

        :param str problemtype: (optional)
            The kind of support request.

        :param str bearertokentype: (optional)
            Token type that determine which cloud provider the request come from.

        :param str bearertoken: (optional)
            Token that provided by multi cloud provider, which help to validate the email.

        :param str idtoken: (optional)
            IdToken that provided by multi cloud provider, which help to validate the email.

        :param str domainid: (optional)
            The OCID of identity domain.
            DomainID is mandatory if the user is part of Non Default Identity domain.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :param bool enable_strict_url_encoding: (optional)
            enable_strict_url_encoding is a boolean to indicate whether or not this request should enable strict url encoding for path params.
            By default, strict url encoding for path params is disabled

        :param int buffer_limit: (optional)
            A buffer limit for the stream to be buffered. buffer_limit is used to set the buffer size capacity. Streams will be read until the size of the buffer reaches the buffer_limit.
            If the stream size is greater than the buffer_limit, a BufferError exception will be thrown.

            The buffer_limit parameter is used when the stream object does not have a `seek`, `tell`, or `fileno` property for the Python Request library to calculate out the content length.
            If buffer_limit is not passed, then the buffer_limit will be defaulted to 100MB.
            Large streams can cause the process to freeze, consider passing in content-length for large streams instead.

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.cims.models.Incident`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.oracle.com/en-us/iaas/tools/python-sdk-examples/latest/cims/put_attachment.py.html>`__ to see an example of how to use put_attachment API.
        """
        # Required path and query arguments. These are in camelCase to replace values in service endpoints.
        required_arguments = ['incidentKey', 'attachmentName', 'compartmentId', 'isRestrictedFlag']
        resource_path = "/v2/incidents/{incidentKey}/attachment"
        method = "PUT"
        operation_name = "put_attachment"
        api_reference_link = "https://docs.oracle.com/iaas/api/#/en/incidentmanagement/20181231/Incident/PutAttachment"

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "enable_strict_url_encoding",
            "retry_strategy",
            "buffer_limit",
            "csi",
            "opc_request_id",
            "ocid",
            "if_match",
            "homeregion",
            "problemtype",
            "bearertokentype",
            "bearertoken",
            "idtoken",
            "domainid"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                f"put_attachment got unknown kwargs: {extra_kwargs!r}")

        path_params = {
            "incidentKey": incident_key
        }

        path_params = {k: v for (k, v) in six.iteritems(path_params) if v is not missing}

        for (k, v) in six.iteritems(path_params):
            if v is None or (isinstance(v, six.string_types) and len(v.strip()) == 0):
                raise ValueError(f'Parameter {k} cannot be None, whitespace or empty string')

        query_params = {
            "attachmentName": attachment_name,
            "compartmentId": compartment_id,
            "isRestrictedFlag": is_restricted_flag
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "csi": kwargs.get("csi", missing),
            "opc-request-id": kwargs.get("opc_request_id", missing),
            "ocid": kwargs.get("ocid", missing),
            "if-match": kwargs.get("if_match", missing),
            "homeregion": kwargs.get("homeregion", missing),
            "problemtype": kwargs.get("problemtype", missing),
            "bearertokentype": kwargs.get("bearertokentype", missing),
            "bearertoken": kwargs.get("bearertoken", missing),
            "idtoken": kwargs.get("idtoken", missing),
            "domainid": kwargs.get("domainid", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        # If the body parameter is optional we need to assign it to a variable so additional type checking can be performed.
        try:
            put_attachment_details
        except NameError:
            put_attachment_details = kwargs.get("put_attachment_details", missing)

        if put_attachment_details is not missing and put_attachment_details is not None:
            if (not isinstance(put_attachment_details, (six.binary_type, six.string_types)) and
                    not hasattr(put_attachment_details, "read")):
                raise TypeError('The body must be a string, bytes, or provide a read() method.')

            if hasattr(put_attachment_details, 'fileno') and hasattr(put_attachment_details, 'name') and put_attachment_details.name != '<stdin>':
                if requests.utils.super_len(put_attachment_details) == 0:
                    header_params['Content-Length'] = '0'

            # If content length is not given and stream object have no 'fileno' and is not a string or bytes, try to calculate content length
            elif 'Content-Length' not in header_params and not is_content_length_calculable_by_req_util(put_attachment_details):
                calculated_obj = back_up_body_calculate_stream_content_length(put_attachment_details, kwargs.get("buffer_limit"))
                header_params['Content-Length'] = calculated_obj["content_length"]
                put_attachment_details = calculated_obj["byte_content"]

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                body=put_attachment_details,
                response_type="Incident",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                body=put_attachment_details,
                response_type="Incident",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)

    def update_incident(self, incident_key, update_incident_details, compartment_id, **kwargs):
        """
        Updates the specified support request.
        For more information, see `Updating Support Requests`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/update-incident.htm


        :param str incident_key: (required)
            Unique identifier for the support request.

        :param oci.cims.models.UpdateIncident update_incident_details: (required)
            Details about the support request being updated.

        :param str compartment_id: (required)
            The OCID of the tenancy.

        :param str csi: (optional)
            The Customer Support Identifier (CSI) number associated with the support account.
            The CSI is optional for all support request types.

        :param str opc_request_id: (optional)
            Unique Oracle-assigned identifier for the request. If you need to contact Oracle about a particular request, please provide the request ID.

        :param str if_match: (optional)
            For optimistic concurrency control. In the PUT or DELETE call for a resource, set the `if-match` parameter to the value of the etag from a previous GET or POST response for that resource. The resource will be updated or deleted only if the etag you provide matches the resource's current etag value.

        :param str ocid: (optional)
            User OCID for Oracle Identity Cloud Service (IDCS) users who also have a federated Oracle Cloud Infrastructure account.
            User OCID is mandatory for OCI Users and optional for Multicloud users.

        :param str homeregion: (optional)
            The region of the tenancy.

        :param str bearertokentype: (optional)
            Token type that determine which cloud provider the request come from.

        :param str bearertoken: (optional)
            Token that provided by multi cloud provider, which help to validate the email.

        :param str idtoken: (optional)
            IdToken that provided by multi cloud provider, which help to validate the email.

        :param str domainid: (optional)
            The OCID of identity domain.
            DomainID is mandatory if the user is part of Non Default Identity domain.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :param bool enable_strict_url_encoding: (optional)
            enable_strict_url_encoding is a boolean to indicate whether or not this request should enable strict url encoding for path params.
            By default, strict url encoding for path params is disabled

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.cims.models.Incident`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.oracle.com/en-us/iaas/tools/python-sdk-examples/latest/cims/update_incident.py.html>`__ to see an example of how to use update_incident API.
        """
        # Required path and query arguments. These are in camelCase to replace values in service endpoints.
        required_arguments = ['incidentKey', 'compartmentId']
        resource_path = "/v2/incidents/{incidentKey}"
        method = "PUT"
        operation_name = "update_incident"
        api_reference_link = "https://docs.oracle.com/iaas/api/#/en/incidentmanagement/20181231/UpdateIncident/UpdateIncident"

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "enable_strict_url_encoding",
            "retry_strategy",
            "csi",
            "opc_request_id",
            "if_match",
            "ocid",
            "homeregion",
            "bearertokentype",
            "bearertoken",
            "idtoken",
            "domainid"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                f"update_incident got unknown kwargs: {extra_kwargs!r}")

        path_params = {
            "incidentKey": incident_key
        }

        path_params = {k: v for (k, v) in six.iteritems(path_params) if v is not missing}

        for (k, v) in six.iteritems(path_params):
            if v is None or (isinstance(v, six.string_types) and len(v.strip()) == 0):
                raise ValueError(f'Parameter {k} cannot be None, whitespace or empty string')

        query_params = {
            "compartmentId": compartment_id
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "csi": kwargs.get("csi", missing),
            "opc-request-id": kwargs.get("opc_request_id", missing),
            "if-match": kwargs.get("if_match", missing),
            "ocid": kwargs.get("ocid", missing),
            "homeregion": kwargs.get("homeregion", missing),
            "bearertokentype": kwargs.get("bearertokentype", missing),
            "bearertoken": kwargs.get("bearertoken", missing),
            "idtoken": kwargs.get("idtoken", missing),
            "domainid": kwargs.get("domainid", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                body=update_incident_details,
                response_type="Incident",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                path_params=path_params,
                query_params=query_params,
                header_params=header_params,
                body=update_incident_details,
                response_type="Incident",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)

    def validate_user(self, **kwargs):
        """
        Checks whether the requested user is valid.
        For more information, see `Validating a User`__.

        __ https://docs.oracle.com/iaas/Content/GSG/support/validate-user.htm


        :param str csi: (optional)
            The Customer Support Identifier (CSI) number associated with the support account.
            The CSI is optional for all support request types.

        :param str opc_request_id: (optional)
            Unique Oracle-assigned identifier for the request. If you need to contact Oracle about a particular request, please provide the request ID.

        :param str problem_type: (optional)
            The kind of support request.

            Allowed values are: "LIMIT", "LEGACY_LIMIT", "TECH", "ACCOUNT", "TAXONOMY"

        :param str ocid: (optional)
            User OCID for Oracle Identity Cloud Service (IDCS) users who also have a federated Oracle Cloud Infrastructure account.
            User OCID is mandatory for OCI Users and optional for Multicloud users.

        :param str homeregion: (optional)
            The region of the tenancy.

        :param str bearertokentype: (optional)
            Token type that determine which cloud provider the request come from.

        :param str bearertoken: (optional)
            Token that provided by multi cloud provider, which help to validate the email.

        :param str idtoken: (optional)
            IdToken that provided by multi cloud provider, which help to validate the email.

        :param str domainid: (optional)
            The OCID of identity domain.
            DomainID is mandatory if the user is part of Non Default Identity domain.

        :param obj retry_strategy: (optional)
            A retry strategy to apply to this specific operation/call. This will override any retry strategy set at the client-level.

            This should be one of the strategies available in the :py:mod:`~oci.retry` module. This operation will not retry by default, users can also use the convenient :py:data:`~oci.retry.DEFAULT_RETRY_STRATEGY` provided by the SDK to enable retries for it.
            The specifics of the default retry strategy are described `here <https://docs.oracle.com/en-us/iaas/tools/python/latest/sdk_behaviors/retries.html>`__.

            To have this operation explicitly not perform any retries, pass an instance of :py:class:`~oci.retry.NoneRetryStrategy`.

        :param bool allow_control_chars: (optional)
            allow_control_chars is a boolean to indicate whether or not this request should allow control characters in the response object.
            By default, the response will not allow control characters in strings

        :param bool enable_strict_url_encoding: (optional)
            enable_strict_url_encoding is a boolean to indicate whether or not this request should enable strict url encoding for path params.
            By default, strict url encoding for path params is disabled

        :return: A :class:`~oci.response.Response` object with data of type :class:`~oci.cims.models.ValidationResponse`
        :rtype: :class:`~oci.response.Response`

        :example:
        Click `here <https://docs.oracle.com/en-us/iaas/tools/python-sdk-examples/latest/cims/validate_user.py.html>`__ to see an example of how to use validate_user API.
        """
        # Required path and query arguments. These are in camelCase to replace values in service endpoints.
        required_arguments = []
        resource_path = "/v2/incidents/user/validate"
        method = "GET"
        operation_name = "validate_user"
        api_reference_link = "https://docs.oracle.com/iaas/api/#/en/incidentmanagement/20181231/ValidationResponse/ValidateUser"

        # Don't accept unknown kwargs
        expected_kwargs = [
            "allow_control_chars",
            "enable_strict_url_encoding",
            "retry_strategy",
            "csi",
            "opc_request_id",
            "problem_type",
            "ocid",
            "homeregion",
            "bearertokentype",
            "bearertoken",
            "idtoken",
            "domainid"
        ]
        extra_kwargs = [_key for _key in six.iterkeys(kwargs) if _key not in expected_kwargs]
        if extra_kwargs:
            raise ValueError(
                f"validate_user got unknown kwargs: {extra_kwargs!r}")

        if 'problem_type' in kwargs:
            problem_type_allowed_values = ["LIMIT", "LEGACY_LIMIT", "TECH", "ACCOUNT", "TAXONOMY"]
            if kwargs['problem_type'] not in problem_type_allowed_values:
                raise ValueError(
                    f"Invalid value for `problem_type`, must be one of { problem_type_allowed_values }"
                )

        query_params = {
            "problemType": kwargs.get("problem_type", missing)
        }
        query_params = {k: v for (k, v) in six.iteritems(query_params) if v is not missing and v is not None}

        header_params = {
            "accept": "application/json",
            "content-type": "application/json",
            "csi": kwargs.get("csi", missing),
            "opc-request-id": kwargs.get("opc_request_id", missing),
            "ocid": kwargs.get("ocid", missing),
            "homeregion": kwargs.get("homeregion", missing),
            "bearertokentype": kwargs.get("bearertokentype", missing),
            "bearertoken": kwargs.get("bearertoken", missing),
            "idtoken": kwargs.get("idtoken", missing),
            "domainid": kwargs.get("domainid", missing)
        }
        header_params = {k: v for (k, v) in six.iteritems(header_params) if v is not missing and v is not None}

        retry_strategy = self.base_client.get_preferred_retry_strategy(
            operation_retry_strategy=kwargs.get('retry_strategy'),
            client_retry_strategy=self.retry_strategy
        )

        if retry_strategy:
            if not isinstance(retry_strategy, retry.NoneRetryStrategy):
                self.base_client.add_opc_client_retries_header(header_params)
                retry_strategy.add_circuit_breaker_callback(self.circuit_breaker_callback)
            return retry_strategy.make_retrying_call(
                self.base_client.call_api,
                resource_path=resource_path,
                method=method,
                query_params=query_params,
                header_params=header_params,
                response_type="ValidationResponse",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)
        else:
            return self.base_client.call_api(
                resource_path=resource_path,
                method=method,
                query_params=query_params,
                header_params=header_params,
                response_type="ValidationResponse",
                allow_control_chars=kwargs.get('allow_control_chars'),
                enable_strict_url_encoding=kwargs.get('enable_strict_url_encoding'),
                operation_name=operation_name,
                api_reference_link=api_reference_link,
                required_arguments=required_arguments)



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class TenancyInformation(object):
    """
    Details about the tenancy.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new TenancyInformation object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param customer_support_key:
            The value to assign to the customer_support_key property of this TenancyInformation.
        :type customer_support_key: str

        :param tenancy_id:
            The value to assign to the tenancy_id property of this TenancyInformation.
        :type tenancy_id: str

        """
        self.swagger_types = {
            'customer_support_key': 'str',
            'tenancy_id': 'str'
        }
        self.attribute_map = {
            'customer_support_key': 'customerSupportKey',
            'tenancy_id': 'tenancyId'
        }
        self._customer_support_key = None
        self._tenancy_id = None

    @property
    def customer_support_key(self):
        """
        **[Required]** Gets the customer_support_key of this TenancyInformation.
        The Customer Support Identifier (CSI) number associated with the tenancy.


        :return: The customer_support_key of this TenancyInformation.
        :rtype: str
        """
        return self._customer_support_key

    @customer_support_key.setter
    def customer_support_key(self, customer_support_key):
        """
        Sets the customer_support_key of this TenancyInformation.
        The Customer Support Identifier (CSI) number associated with the tenancy.


        :param customer_support_key: The customer_support_key of this TenancyInformation.
        :type: str
        """
        self._customer_support_key = customer_support_key

    @property
    def tenancy_id(self):
        """
        **[Required]** Gets the tenancy_id of this TenancyInformation.
        The OCID of the tenancy.


        :return: The tenancy_id of this TenancyInformation.
        :rtype: str
        """
        return self._tenancy_id

    @tenancy_id.setter
    def tenancy_id(self, tenancy_id):
        """
        Sets the tenancy_id of this TenancyInformation.
        The OCID of the tenancy.


        :param tenancy_id: The tenancy_id of this TenancyInformation.
        :type: str
        """
        self._tenancy_id = tenancy_id

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class IncidentType(object):
    """
    Details about the incident type associated with the support request.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new IncidentType object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param id:
            The value to assign to the id property of this IncidentType.
        :type id: str

        :param name:
            The value to assign to the name property of this IncidentType.
        :type name: str

        :param label:
            The value to assign to the label property of this IncidentType.
        :type label: str

        :param description:
            The value to assign to the description property of this IncidentType.
        :type description: str

        :param classifier_list:
            The value to assign to the classifier_list property of this IncidentType.
        :type classifier_list: list[oci.cims.models.Classifier]

        """
        self.swagger_types = {
            'id': 'str',
            'name': 'str',
            'label': 'str',
            'description': 'str',
            'classifier_list': 'list[Classifier]'
        }
        self.attribute_map = {
            'id': 'id',
            'name': 'name',
            'label': 'label',
            'description': 'description',
            'classifier_list': 'classifierList'
        }
        self._id = None
        self._name = None
        self._label = None
        self._description = None
        self._classifier_list = None

    @property
    def id(self):
        """
        Gets the id of this IncidentType.
        Unique identifier for the incident type.


        :return: The id of this IncidentType.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this IncidentType.
        Unique identifier for the incident type.


        :param id: The id of this IncidentType.
        :type: str
        """
        self._id = id

    @property
    def name(self):
        """
        Gets the name of this IncidentType.
        The name of the incident type.


        :return: The name of this IncidentType.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this IncidentType.
        The name of the incident type.


        :param name: The name of this IncidentType.
        :type: str
        """
        self._name = name

    @property
    def label(self):
        """
        Gets the label of this IncidentType.
        The label associated with the incident type.


        :return: The label of this IncidentType.
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """
        Sets the label of this IncidentType.
        The label associated with the incident type.


        :param label: The label of this IncidentType.
        :type: str
        """
        self._label = label

    @property
    def description(self):
        """
        Gets the description of this IncidentType.
        The description of the incident type.


        :return: The description of this IncidentType.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this IncidentType.
        The description of the incident type.


        :param description: The description of this IncidentType.
        :type: str
        """
        self._description = description

    @property
    def classifier_list(self):
        """
        Gets the classifier_list of this IncidentType.
        The list of classifiers.


        :return: The classifier_list of this IncidentType.
        :rtype: list[oci.cims.models.Classifier]
        """
        return self._classifier_list

    @classifier_list.setter
    def classifier_list(self, classifier_list):
        """
        Sets the classifier_list of this IncidentType.
        The list of classifiers.


        :param classifier_list: The classifier_list of this IncidentType.
        :type: list[oci.cims.models.Classifier]
        """
        self._classifier_list = classifier_list

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class SubCategories(object):
    """
    List of subcategories under a service.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new SubCategories object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param service_category:
            The value to assign to the service_category property of this SubCategories.
        :type service_category: dict(str, str)

        :param schema:
            The value to assign to the schema property of this SubCategories.
        :type schema: str

        :param has_sub_category:
            The value to assign to the has_sub_category property of this SubCategories.
        :type has_sub_category: str

        :param sub_categories:
            The value to assign to the sub_categories property of this SubCategories.
        :type sub_categories: list[oci.cims.models.SubComponents]

        """
        self.swagger_types = {
            'service_category': 'dict(str, str)',
            'schema': 'str',
            'has_sub_category': 'str',
            'sub_categories': 'list[SubComponents]'
        }
        self.attribute_map = {
            'service_category': 'serviceCategory',
            'schema': 'schema',
            'has_sub_category': 'hasSubCategory',
            'sub_categories': 'subCategories'
        }
        self._service_category = None
        self._schema = None
        self._has_sub_category = None
        self._sub_categories = None

    @property
    def service_category(self):
        """
        Gets the service_category of this SubCategories.
        Subcategory list.


        :return: The service_category of this SubCategories.
        :rtype: dict(str, str)
        """
        return self._service_category

    @service_category.setter
    def service_category(self, service_category):
        """
        Sets the service_category of this SubCategories.
        Subcategory list.


        :param service_category: The service_category of this SubCategories.
        :type: dict(str, str)
        """
        self._service_category = service_category

    @property
    def schema(self):
        """
        Gets the schema of this SubCategories.
        Schema of a subcategory.


        :return: The schema of this SubCategories.
        :rtype: str
        """
        return self._schema

    @schema.setter
    def schema(self, schema):
        """
        Sets the schema of this SubCategories.
        Schema of a subcategory.


        :param schema: The schema of this SubCategories.
        :type: str
        """
        self._schema = schema

    @property
    def has_sub_category(self):
        """
        Gets the has_sub_category of this SubCategories.
        Flag to identify if subComponent is present


        :return: The has_sub_category of this SubCategories.
        :rtype: str
        """
        return self._has_sub_category

    @has_sub_category.setter
    def has_sub_category(self, has_sub_category):
        """
        Sets the has_sub_category of this SubCategories.
        Flag to identify if subComponent is present


        :param has_sub_category: The has_sub_category of this SubCategories.
        :type: str
        """
        self._has_sub_category = has_sub_category

    @property
    def sub_categories(self):
        """
        Gets the sub_categories of this SubCategories.
        The sub component list for MOS Taxonomy.


        :return: The sub_categories of this SubCategories.
        :rtype: list[oci.cims.models.SubComponents]
        """
        return self._sub_categories

    @sub_categories.setter
    def sub_categories(self, sub_categories):
        """
        Sets the sub_categories of this SubCategories.
        The sub component list for MOS Taxonomy.


        :param sub_categories: The sub_categories of this SubCategories.
        :type: list[oci.cims.models.SubComponents]
        """
        self._sub_categories = sub_categories

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2026, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181231


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class CmosUserGroupInfo(object):
    """
    Identifier and name of a user group (`userGroupId` and `userGroupName`) that the current user has access to.
    Note: The Customer User Administrator (CUA) can manage user groups by name using
    `My Oracle Cloud Support portal`__.

    __ https://support.oracle.com
    """

    def __init__(self, **kwargs):
        """
        Initializes a new CmosUserGroupInfo object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param user_group_id:
            The value to assign to the user_group_id property of this CmosUserGroupInfo.
        :type user_group_id: str

        :param user_group_name:
            The value to assign to the user_group_name property of this CmosUserGroupInfo.
        :type user_group_name: str

        """
        self.swagger_types = {
            'user_group_id': 'str',
            'user_group_name': 'str'
        }
        self.attribute_map = {
            'user_group_id': 'userGroupId',
            'user_group_name': 'userGroupName'
        }
        self._user_group_id = None
        self._user_group_name = None

    @property
    def user_group_id(self):
        """
        Gets the user_group_id of this CmosUserGroupInfo.
        Technical support type (`TECH`) only: The identifier of a user group in My Oracle Cloud Support portal.


        :return: The user_group_id of this CmosUserGroupInfo.
        :rtype: str
        """
        return self._user_group_id

    @user_group_id.setter
    def user_group_id(self, user_group_id):
        """
        Sets the user_group_id of this CmosUserGroupInfo.
        Technical support type (`TECH`) only: The identifier of a user group in My Oracle Cloud Support portal.


        :param user_group_id: The user_group_id of this CmosUserGroupInfo.
        :type: str
        """
        self._user_group_id = user_group_id

    @property
    def user_group_name(self):
        """
        Gets the user_group_name of this CmosUserGroupInfo.
        Technical support type (`TECH`) only: Name of a user group in My Oracle Cloud Support portal.


        :return: The user_group_name of this CmosUserGroupInfo.
        :rtype: str
        """
        return self._user_group_name

    @user_group_name.setter
    def user_group_name(self, user_group_name):
        """
        Sets the user_group_name of this CmosUserGroupInfo.
        Technical support type (`TECH`) only: Name of a user group in My Oracle Cloud Support portal.


        :param user_group_name: The user_group_name of this CmosUserGroupInfo.
        :type: str
        """
        self._user_group_name = user_group_name

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
